/*
 * Soubor:  proj2.c
 * Datum:   01.08.2004 21:45
 * Autor:   David Martinek, martinek@fit.vutbr.cz
 * Projekt: Datumy, projekt . 2 pro pedmt IZP
 * Popis:   Program provd vpoet rozdlu mezi dvmi kalendnmi daty.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdbool.h>
#include <limits.h>


const unsigned int MAX_YEAR = (unsigned int)(UINT_MAX / 365.25);
// #define MAX_YEAR 11000000


/**
 * Vytiskne na standardn vstup text s npovdou.
 */
void printHelp()
{
  printf("Program Datumy - vzorov implementace.\n"
         "Autor: David Martinek (c) 2004\n"
         "Program pot rozdl zadanch kalendnch dat.\n"
         "Data jsou natna ze standardnho vstupu ve formtu\n"
         "dd.mm.rrrr-dd.mm.rrrr\n"
         "Rozsah povolench let je 1 - %d\n"
         "Popis parametr:\n"
         "  -h  Vype tuto obrazovku s npovdou.\n", MAX_YEAR);
}

/**
 * Zpracuje argumenty pkazovho dku. Pokud naraz na argument -h, vype
 * npovdu a ukon program.
 * @param argc Poet argument.
 * @param argv Pole textovch etzc s argumenty.
 */
void doParams(int argc, char *argv[])
{
  if (argc == 2 && strcmp("-h", argv[1]) == 0)
  { // tisk npovdy
    printHelp();
    exit(EXIT_SUCCESS); // s tmto opatrn!
  }
}

/**
 * Struktura pro reprezentaci jedinho data.
 */
typedef struct tdate
{
  unsigned int day;
  unsigned int month;
  unsigned int year;
} TDate;

/**
 * Vrac true, pokud je parametr pestupn rok.
 */
bool isLeapYear(unsigned int year)
{
  return (year % 4 == 0) &&
         ((year % 100 != 0) || (year % 400 == 0));
}

/**
 * Vrac true, pokud je hodnota datumu v podku.
 */
bool isDateOk(const TDate *date)
{
  bool ok = date->month > 0 && date->month <= 12 &&
            date->year > 0 && date->year <= MAX_YEAR;
  if (!ok) return false;

  unsigned int maxDay = 31;
  if (isLeapYear(date->year)) 
  {
    if (date->month == 2) maxDay = 29;
  }
  else
  {
    if (date->month == 2) maxDay = 28;
  }

  if (date->month == 4 || date->month == 6 || 
      date->month == 9 || date->month == 11)
  {
    maxDay = 30;
  }

  ok = date->day > 0 && date->day <= maxDay;
  return ok;
  // Poznmka: Normln nejsem zastncem "magickch konstant", ale v tto funkci
  // jde o natolik bn pouvan hodnoty, kter se navc nikdy nebudou mnit 
  // (leda by pi dalm pevratu nkdo pesloval msce), e by tady pojmenovan
  // konstanty nepinesly dnou zsadn vhodu. Jinak ale plat, e 
}

/**
 * Vrac poet dn od zatku letopotu do zatku zadanho roku (pit i rok
 * "0", ale potom se pot rozdl, take to nehraje roli).
 */
unsigned int getDaysToYear(unsigned int year)
{
  year -= 1;  //aktuln rok nechci zapotat, ten se zapot jinde
  return year*365 + year/4 - year/100 + year/400;
}

/**
 * Vrt poet dn od zatku letopotu do zadanho data.
 */
unsigned int getDays(const TDate *date)
{
  static const unsigned int daysToMonth[12] =
  {
    0, 
    31, 
    31+28, 
    31+28+31, 
    31+28+31+30,
    31+28+31+30+31,
    31+28+31+30+31+30,
    31+28+31+30+31+30+31,
    31+28+31+30+31+30+31+31,
    31+28+31+30+31+30+31+31+30,
    31+28+31+30+31+30+31+31+30+31,
    31+28+31+30+31+30+31+31+30+31+30
  };
  unsigned int days = getDaysToYear(date->year) + daysToMonth[date->month-1] 
    + date->day;
  
  if (isLeapYear(date->year) && date->month > 2)
  {
    days += 1;
  }

  return days;
}

/** Kdy chyb, kter se mohou vyskytnout ve funkci readDates. */
enum terrors
{
  EOK,      /**< Bez chyby. */
  EFORMAT,  /**< Chyba ve formtu vstupnho vrazu. */
  EDATE1,   /**< Nesmysln hodnota prvnho datumu. */
  EDATE2,   /**< Nesmysln hodnota druhho datumu. */
  EUNKNOWN  /**< Neznm chyba. */
};

/** 
 * Chybov hlen. Jejich poad v poli odpovd poad konstant
 * ve vtu terrors.
 */
const char *EMSG[] =
{
  "Ve v podku.",
  "Chybn formt vstupnho vrazu.",
  "Prvn datum m nesmyslnou hodnotu.",
  "Druh datum m nesmyslnou hodnotu.",
  "Neznm chyba! Nco je tragicky patn!"
};

/**
 * Vytiskne chybov hlen na chybov vstup. 
 */
void printError(int error)
{
  if (error < EOK || error >= EUNKNOWN) 
    error = EUNKNOWN;
  fprintf(stderr, "Chyba: %s\n", EMSG[error]);
}


/**
 * Pete vraz ve vstupnm streamu a pevede jej na dva datumy. Pokud
 * je vraz zadn chybn, vrt chybov kd.
 * @param stream Vstupn stream (soubor nebo stdin) ze kterho se bude st.
 * @param date1 Prvn datum nalezen v textovm etzci. (Pedvn odkazem.)
 * @param date2 Druh datum nalezen v textovm etzci. (Pedvn odkazem.)
 * @return Vrac chybov kd z vtu enum terrors.
 */
int readDates(FILE *stream, TDate *date1, TDate *date2)
{
  assert(stream != NULL && date1 != NULL && date2 != NULL);

  int num = fscanf(stream, "%u.%u.%u-%u.%u.%u", 
      &date1->day, &date1->month, &date1->year,
      &date2->day, &date2->month, &date2->year);

  if (num != 6) return EFORMAT; // oekvm, e fscanf zpracovala 6 poloek
  if (!isDateOk(date1)) return EDATE1;
  if (!isDateOk(date2)) return EDATE2;
  return EOK;
}

/**
 * Odete dv data a vrt poet dn mezi tmito datumy.
 */
unsigned int getDiff(const TDate *date1, const TDate *date2)
{
  unsigned int d1 = getDays(date1);
  unsigned int d2 = getDays(date2);
  return (d1 < d2)? d2 - d1 : d1 - d2;
}


/////////////////////////////////////////////////////
/**
 * Hlavn program.
 */
int main(int argc, char *argv[])
{
  doParams(argc, argv);

  TDate date1, date2;
  int err;

  if ((err = readDates(stdin, &date1, &date2)) != EOK)
  {
    fprintf(stderr, "Chyba pi natn vstupnch dat!\n");
    printError(err);
    return EXIT_FAILURE;
  }

//  printf("Datum1: %d.%d.%d\n", date1.day, date1.month, date1.year);
//  printf("Datum2: %d.%d.%d\n", date2.day, date2.month, date2.year);
//  printf("Rozdl: %u\n", getDiff(date1, date2));
  printf("%u\n", getDiff(&date1, &date2));

  return EXIT_SUCCESS;
}

/* konec souboru proj2.c */
